#!/usr/bin/env python
# coding: utf-8
try:
      import numpy as np
      from scipy import stats
      import random
      import momi
except:
      pass
import os
import sys

print("Do not run this without checking, this will write files and overwrite things")
sys.exit(-1)

os.environ['KMP_DUPLICATE_LIB_OK']='True'
import matplotlib.pyplot as plt
mypython = '/usr/local/anaconda3/bin/python'
####    os.system("python manyloci.py ms 40 "+loci+" -t 20 -r 0 1000 -I 2 20 20 0.25 -n 1 0.5 -n 2 0.5 -T -M -q 1000 -ej "+tmrca+"  2 1 -eN "+tmrca+" 0.5 > tree; migdata --modern < tree; cp infile "+infile)

mu =  5.0e-7
ne = 10000
recoms_per_gen = 1.0e-8
bases_per_locus = int(1e6)
n_loci = 1
ploidy = 2
split=100000
TLIMIT=1
simdir = f'sim2{ne}_{n_loci}_{split}'

model = momi.DemographicModel(N_e=ne, gen_time=1, muts_per_gen=mu)

model.add_leaf("YRI", N=5e3)
model.add_leaf("CHB", N=5e3)

# at 85 kya CHB joins onto YRI; YRI is set to size N=1.2e4
model.move_lineages("CHB", "YRI", t=split, N=ne//2)

yticks = [1e4, 2.5e4, 5e4, 7.5e4, 1e5, 2.5e5, 5e5, 7.5e5]
fig = momi.DemographyPlot(
    model, ["YRI", "CHB"],
    figsize=(6,8),
    major_yticks=yticks,
    linthreshy=1e5, pulse_color_bounds=(0,.25))
plt.show()


# n_alleles per population (n_individuals = n_alleles / ploidy)
sampled_n_dict = {"YRI":10, "CHB":10}

# create data directory if it doesn't exist
os.system(f'mkdir -p {simdir}/')

# simulate 10 "chromosomes", saving each in a separate vcf file
for chrom in range(1, n_loci+1):
      seed = random.randint(0,32000)
      model.simulate_vcf(
            f"{simdir}/{chrom}",
            recoms_per_gen=recoms_per_gen,
            length=bases_per_locus,
            chrom_name=f"chr{chrom}",
            ploidy=ploidy,
            random_seed=seed,
            sampled_n_dict=sampled_n_dict,
            force=True)


# a dict mapping samples to populations
ind2pop = {}
for pop, n in sampled_n_dict.items():
    for i in range(int(n / ploidy)):
        # in the vcf, samples are named like YRI_0, YRI_1, CHB_0, etc
        ind2pop["{}_{}".format(pop, i)] = pop

with open(f"{simdir}/ind2pop.txt", "w") as f:
    for i, p in ind2pop.items():
        print(i, p, sep="\t", file=f)

#sys.exit()
        
for chrom in range(1,n_loci+1):
      os.system(f"{mypython} -m momi.read_vcf {simdir}/{chrom}.vcf.gz {simdir}/ind2pop.txt {simdir}/{chrom}.snpAlleleCounts.gz --bed {simdir}/{chrom}.bed")

os.system(f'{mypython} -m momi.extract_sfs {simdir}/sfs.gz 100 {simdir}/*.snpAlleleCounts.gz')


sfs = momi.Sfs.load(f"{simdir}/sfs.gz")


no_pulse_model = momi.DemographicModel(
      N_e=ne, gen_time=1, muts_per_gen=mu)

no_pulse_model.set_data(sfs)

no_pulse_model.add_size_param("n_chb")
no_pulse_model.add_size_param("n_yri")
no_pulse_model.add_time_param("t_chb_yri", lower=TLIMIT)

no_pulse_model.add_leaf("CHB", N="n_chb")
no_pulse_model.add_leaf("YRI", N="n_yri")
no_pulse_model.move_lineages("CHB", "YRI", t="t_chb_yri", N=ne//2)

no_pulse_model.optimize(method="TNC")


params = no_pulse_model.get_params()
print(f"Parameter estimated: {params}")
fig = momi.DemographyPlot(no_pulse_model, ["YRI", "CHB"],
                          figsize=(6,8), linthreshy=1e5,
                          major_yticks=yticks,
                          pulse_color_bounds=(0,.25))
n_bootstraps = 100
# make copies of the original models to avoid changing them
no_pulse_copy = no_pulse_model.copy()
#add_pulse_copy = add_pulse_model.copy()

les = []
bootstrap_results=[]

for i in range(0,n_bootstraps):
      #print(f"Fitting {i+1}-th bootstrap out of {n_bootstraps}")
      print(i,end=' ')
      resampled_sfs = sfs.resample()
      no_pulse_copy.set_data(resampled_sfs)
      no_pulse_copy.set_params(randomize=True)
      no_pulse_copy.optimize()
      bootstrap_results.append(no_pulse_copy.get_params())


# make canvas, but delay plotting the demography (draw=False)
fig = momi.DemographyPlot(
    no_pulse_model, ["YRI", "CHB"],
    linthreshy=1e5, figsize=(6,8),
    major_yticks=yticks,
    draw=False)

# plot bootstraps onto the canvas in transparency
for params in bootstrap_results:
    fig.add_bootstrap(
        params,
        # alpha=0: totally transparent. alpha=1: totally opaque
        alpha=1/n_bootstraps)

# now draw the inferred demography on top of the bootstraps
fig.draw()
fig.draw_N_legend(loc="upper left")
plt.show()

z1=[bootstrap_results[i]['n_chb'] for i in range(len(bootstrap_results))]
z2=[bootstrap_results[i]['n_yri'] for i in range(len(bootstrap_results))]
z3=[bootstrap_results[i]['t_chb_yri'] for i in range(len(bootstrap_results))]


z = [z1,z2,z3]
z = np.array(z)

modez= [stats.describe(zi) for zi in z]
for i in modez:
      print(i)


z1.sort()
_ = plt.hist(z1[5:-5],20,density=0.9)
z2.sort()
_ = plt.hist(z2[5:-5],20,density=0.9,alpha=0.5)
plt.show()
z3.sort()
_ = plt.hist(z3[5:-5],20,density=0.9,alpha=0.5)
plt.show()

